from typing import Dict, List, Optional, Union
from datetime import datetime
from pydantic import BaseModel, Field, validator
from .utils import clamp


class RateLimit(BaseModel):
    limit: int = Field(alias="X-RateLimit-Limit")
    remaining: int = Field(alias="X-RateLimit-Remaining")
    reset: int = Field(alias="X-RateLimit-Reset")

    @property
    def reset_datetime(self):
        return datetime.fromtimestamp(self.reset)

    @property
    def reset_timedelta(self):
        return self.reset_datetime - datetime.now()


class BotMe(BaseModel):
    id: str
    priority: int
    concurrency: int
    quota: int
    quota_used: int
    limits: RateLimit


class AniList(BaseModel):
    id: int
    id_mal: int = Field(alias="idMal")
    title: Dict[str, Optional[str]]
    synonyms: List[str]
    is_adult: bool = Field(alias="isAdult")

    @property
    def mal_url(self) -> str:
        return f"https://myanimelist.net/anime/{self.id_mal}"


class AnimeSearch(BaseModel):
    anilist: Union[int, AniList]
    filename: str
    episode: Union[int, str, None]
    anime_from: float = Field(alias="from")
    anime_to: float = Field(alias="to")
    similarity: float
    video: str
    image: str

    def short_similarity(self, formatting="{:.1%}", *args) -> str:
        return clamp(self.similarity, format=formatting, *args)


class AnimeResponse(BaseModel):
    frame_count: int = Field(alias="frameCount")
    error: Optional[str]
    result: List[AnimeSearch]
    limits: RateLimit

    @validator("error")
    def has_error(cls, error):
        if error != "":
            return error

    @property
    def best_result(self) -> AnimeSearch:
        return self.result[0]



#!/usr/bin/env python3
import pathlib
import re
import sys

from setuptools import find_packages, setup

WORK_DIR = pathlib.Path(__file__).parent

# Check python version
MINIMAL_PY_VERSION = (3, 7)
if sys.version_info < MINIMAL_PY_VERSION:
    raise RuntimeError('aiotracemoeapi works only with Python {}+'.format('.'.join(map(str, MINIMAL_PY_VERSION))))


def get_version():
    """
    Read version

    :return: str
    """
    txt = (WORK_DIR / 'aiotracemoeapi' / '__init__.py').read_text('utf-8')
    try:
        return re.findall(r"^__version__ = '([^']+)'\r?$", txt, re.M)[0]
    except IndexError:
        raise RuntimeError('Unable to determine version.')


setup(
    name='aiotracemoeapi',
    version=get_version(),
    packages=find_packages(exclude=('examples.*',)),
    url='https://github.com/Fenicu/AioTraceMoeAPI',
    license='MIT',
    author='Fenicu',
    python_requires='>=3.7',
    author_email='Fenicu@fenicu.men',
    description='Is a pretty simple and fully asynchronous wrapper for Trace.Moe API',
    classifiers=[
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.7',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Topic :: Software Development :: Libraries :: Application Frameworks',
    ],
    install_requires=[
        'aiohttp>=3.7.2,<4.0.0',
        'yarl==1.4.2',
        'pydantic>=1.8.2'
    ],
    include_package_data=False,
)
